@@ -2,29 +2,27 @@
 # coding: utf-8
 
 # FIXME python2
-from __future__ import unicode_literals
-from __future__ import print_function
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 from future.utils import python_2_unicode_compatible
 
 import base64
 import logging
 import os
 import re
-from uuid import UUID
-from io import BytesIO
-from pykeepass.kdbx_parsing.kdbx import KDBX
-from pykeepass.kdbx_parsing.kdbx4 import kdf_uuids
+import uuid
+import zlib
+
+from construct import Container, ChecksumError
 from lxml import etree
 from lxml.builder import E
-import zlib
-from construct import Container
 
-from pykeepass.entry import Entry
-from pykeepass.xpath import entry_xp, group_xp, attachment_xp, path_xp
-from pykeepass.group import Group
 from pykeepass.attachment import Attachment
+from pykeepass.entry import Entry
 from pykeepass.exceptions import *
+from pykeepass.group import Group
+from pykeepass.kdbx_parsing.kdbx import KDBX
+from pykeepass.kdbx_parsing.kdbx4 import kdf_uuids
+from pykeepass.xpath import attachment_xp, entry_xp, group_xp, path_xp
 
 logger = logging.getLogger(__name__)
 
@@ -46,7 +44,8 @@ class PyKeePass(object):
         return self
 
     def __exit__(self, typ, value, tb):
-        del self.kdbx
+        # see issue 137
+        pass
 
     def read(self, filename=None, password=None, keyfile=None,
              transformed_key=None):
@@ -55,13 +54,15 @@ class PyKeePass(object):
         if not filename:
             filename = self.filename
 
-        self.kdbx = KDBX.parse_file(
-            filename,
-            password=password,
-            keyfile=keyfile,
-            transformed_key=transformed_key
-        )
-
+        try:
+            self.kdbx = KDBX.parse_file(
+                filename,
+                password=password,
+                keyfile=keyfile,
+                transformed_key=transformed_key
+            )
+        except ChecksumError:
+            raise CredentialsIntegrityError
 
     def save(self, filename=None, transformed_key=None):
         if not filename:
@@ -119,6 +120,14 @@ class PyKeePass(object):
     def entries(self):
         return self.find_entries_by_title('.*', regex=True)
 
+    def xml(self):
+        return etree.tostring(
+            self.tree,
+            pretty_print=True,
+            standalone=True,
+            encoding='unicode'
+        )
+
     def dump_xml(self, outfile):
         '''
         Dump the content of the database to a file
@@ -165,47 +174,53 @@ class PyKeePass(object):
 
         return res
 
-
     def _find(self, prefix, keys_xp, path=None, tree=None, first=False,
               history=False, regex=False, flags=None, **kwargs):
 
         xp = ''
 
-        # resolve path before handling any keys
         if path is not None:
 
+            first = True
+
             xp += '/KeePassFile/Root/Group'
-            # split provided path into group and entry
-            group_path = os.path.dirname(path).lstrip('/')
-            entry = os.path.basename(path)
-            # build xpath from group_path and entry
+            path = path.strip('/')
+            # split provided path into group and element
+            group_path = os.path.dirname(path)
+            element = os.path.basename(path)
+            # build xpath from group_path and element
             if group_path:
                 for group in group_path.split('/'):
                     xp += path_xp[regex]['group'].format(group, flags=flags)
-            if entry:
-                xp += path_xp[regex]['entry'].format(entry, flags=flags)
+            if 'Entry' in prefix:
+                xp += path_xp[regex]['entry'].format(element, flags=flags)
+            elif element and 'Group' in prefix:
+                xp += path_xp[regex]['group'].format(element, flags=flags)
 
-        elif tree is not None:
-            xp += '.'
-            
-        # FIXME: ideally, this should be 'if kwargs.keys() or path is not None'
-        #        but this is a breaking change. Pending issue 127
-        if kwargs.keys():
-            xp += prefix
+        else:
+            if tree is not None:
+                xp += '.'
 
-        # handle searching custom string fields
-        if 'string' in kwargs.keys():
-            for key, value in kwargs['string'].items():
-                xp += keys_xp[regex]['string'].format(key, value, flags=flags)
+            if kwargs.keys():
+                xp += prefix
 
-            kwargs.pop('string')
+            # handle searching custom string fields
+            if 'string' in kwargs.keys():
+                for key, value in kwargs['string'].items():
+                    xp += keys_xp[regex]['string'].format(key, value, flags=flags)
 
-        # build xpath to filter results with specified attributes
-        for key, value in kwargs.items():
-            if key not in keys_xp[regex].keys():
-                raise TypeError('Invalid keyword argument "{}"'.format(key))
+                kwargs.pop('string')
 
-            xp += keys_xp[regex][key].format(value, flags=flags)
+            # convert uuid to base64 form before building xpath
+            if 'uuid' in kwargs.keys():
+                kwargs['uuid'] = base64.b64encode(kwargs['uuid'].bytes).decode('utf-8')
+
+            # build xpath to filter results with specified attributes
+            for key, value in kwargs.items():
+                if key not in keys_xp[regex].keys():
+                    raise TypeError('Invalid keyword argument "{}"'.format(key))
+
+                xp += keys_xp[regex][key].format(value, flags=flags)
 
         res = self._xpath(
             xp,
@@ -218,52 +233,36 @@ class PyKeePass(object):
 
         return res
 
-    #---------- Groups ----------
+    # ---------- Groups ----------
 
     def find_groups(self, recursive=True, path=None, group=None, **kwargs):
 
         prefix = '//Group' if recursive else '/Group'
         res = self._find(prefix, group_xp, path=path, tree=group, **kwargs)
-
-
         return res
 
-
-    def find_groups_by_path(self, group_path_str=None, regex=False, flags=None,
-                            group=None, first=False):
-
-        return self.find_groups(name=group_name,
-                                regex=regex,
-                                flags=flags,
-                                group=group,
-                                first=first
-        )
-
-
     def find_groups_by_name(self, group_name, regex=False, flags=None,
                             group=None, first=False):
-
-        return self.find_groups(name=group_name,
-                                regex=regex,
-                                flags=flags,
-                                group=group,
-                                first=first
+        return self.find_groups(
+            name=group_name,
+            regex=regex,
+            flags=flags,
+            group=group,
+            first=first
         )
 
-
     def find_groups_by_path(self, group_path_str=None, regex=False, flags=None,
                             group=None, first=False):
-
-        return self.find_groups(path=group_path_str,
-                                regex=regex,
-                                flags=flags,
-                                group=group,
-                                first=first
+        return self.find_groups(
+            path=group_path_str,
+            regex=regex,
+            flags=flags,
+            group=group,
+            first=first
         )
 
     def find_groups_by_uuid(self, uuid, regex=False, flags=None,
-                              group=None, history=False, first=False):
-
+                            group=None, history=False, first=False):
         return self.find_groups(
             uuid=uuid,
             regex=regex,
@@ -274,8 +273,7 @@ class PyKeePass(object):
         )
 
     def find_groups_by_notes(self, notes, regex=False, flags=None,
-                              group=None, history=False, first=False):
-
+                             group=None, history=False, first=False):
         return self.find_groups(
             notes=notes,
             regex=regex,
@@ -300,10 +298,33 @@ class PyKeePass(object):
     def delete_group(self, group):
         group.delete()
 
+    def deref(self, value):
+        if not value:
+            return value
+        references = set(re.findall(r'({REF:([TUPANI])@([TUPANI]):([^}]+)})', value))
+        if not references:
+            return value
+        field_to_attribute = {
+            'T': 'title',
+            'U': 'username',
+            'P': 'password',
+            'A': 'url',
+            'N': 'notes',
+            'I': 'uuid',
+        }
+        for ref, wanted_field, search_in, search_value in references:
+            wanted_field = field_to_attribute[wanted_field]
+            search_in = field_to_attribute[search_in]
+            if search_in == 'uuid':
+                search_value = uuid.UUID(search_value)
+            ref_entry = self.find_entries(first=True, **{search_in: search_value})
+            value = value.replace(ref, getattr(ref_entry, wanted_field))
+        return self.deref(value)
+
     def move_group(self, group, destination_group):
         destination_group.append(group)
 
-    #---------- Entries ----------
+    # ---------- Entries ----------
 
     def find_entries(self, recursive=True, path=None, group=None, **kwargs):
 
@@ -312,7 +333,6 @@ class PyKeePass(object):
 
         return res
 
-
     def find_entries_by_title(self, title, regex=False, flags=None,
                               group=None, history=False, first=False):
         return self.find_entries(
@@ -380,7 +400,7 @@ class PyKeePass(object):
         )
 
     def find_entries_by_uuid(self, uuid, regex=False, flags=None,
-                              group=None, history=False, first=False):
+                             group=None, history=False, first=False):
         return self.find_entries(
             uuid=uuid,
             regex=regex,
@@ -391,7 +411,7 @@ class PyKeePass(object):
         )
 
     def find_entries_by_string(self, string, regex=False, flags=None,
-                              group=None, history=False, first=False):
+                               group=None, history=False, first=False):
         return self.find_entries(
             string=string,
             regex=regex,
@@ -443,7 +463,7 @@ class PyKeePass(object):
     def move_entry(self, entry, destination_group):
         destination_group.append(entry)
 
-    #---------- Attachments ----------
+    # ---------- Attachments ----------
 
     def find_attachments(self, recursive=True, path=None, element=None, **kwargs):
 
@@ -454,26 +474,26 @@ class PyKeePass(object):
 
     @property
     def attachments(self):
-        self.find_attachments(filename='.*', regex=True)
+        return self.find_attachments(filename='.*', regex=True)
 
     @property
     def binaries(self):
         if self.version >= (4, 0):
             # first byte is a prepended flag
-            attachments = [a.data[1:] for a in self.kdbx.body.payload.inner_header.binary]
+            binaries = [a.data[1:] for a in self.kdbx.body.payload.inner_header.binary]
         else:
-            attachments = []
+            binaries = []
             for elem in self._xpath('/KeePassFile/Meta/Binaries/Binary'):
-                if elem.attrib['Compressed'] == 'True':
+                if elem.get('Compressed') == 'True':
                     data = zlib.decompress(
                         base64.b64decode(elem.text),
                         zlib.MAX_WBITS | 32
                     )
                 else:
                     data = base64.b64decode(elem.text)
-                attachments.append(data)
+                binaries.insert(int(elem.attrib['ID']), data)
 
-        return attachments
+        return binaries
 
     def add_binary(self, data, compressed=True, protected=True):
         if self.version >= (4, 0):
@@ -492,16 +512,25 @@ class PyKeePass(object):
             )
             if compressed:
                 # gzip compression
-                data = zlib.compress(data)
-            data = base64.b64encode(data).decode
+                compressor = zlib.compressobj(
+                    zlib.Z_DEFAULT_COMPRESSION,
+                    zlib.DEFLATED,
+                    zlib.MAX_WBITS | 16
+                )
+                data = compressor.compress(data)
+                data += compressor.flush()
+            data = base64.b64encode(data).decode()
+
+            # set ID for Binary Element
+            binary_id = len(self.binaries)
 
             # add binary element to XML
             binaries.append(
-                E.Binary(data, Compressed=str(compressed))
+                E.Binary(data, ID=str(binary_id), Compressed=str(compressed))
             )
 
-        # return attachment id
-        return len(self.binaries)
+        # return binary id
+        return len(self.binaries) - 1
 
     def delete_binary(self, id):
         try:
@@ -513,7 +542,7 @@ class PyKeePass(object):
                 binaries = self._xpath('/KeePassFile/Meta/Binaries', first=True)
                 binaries.remove(binaries.getchildren()[id])
         except IndexError:
-            raise AttachmentError('No such attachment with id {}'.format(id))
+            raise BinaryError('No such binary with id {}'.format(id))
 
         # remove all entry references to this attachment
         for reference in self.find_attachments(id=id):
